/* 
 * tkMacConsole.c --
 *
 *	This file contains Macintosh-specific procedures for the notifier,
 *	which is the lowest-level part of the Tcl event loop.  This file
 *	works together with ../generic/tclNotify.c.
 *
 * Copyright (c) 1995-1996 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

static char sccsid[] = "@(#) tkMacConsole.c 1.8 96/01/25 14:49:54";

#ifndef __CONSOLE__
#include <console.h>
#endif

#include <Events.h>
#include <SIOUX.h> 
#include <GUSI.h> 

#include "tcl.h"
#include "tk.h"
#include "tkMacInt.h"

Tcl_Interp *gStdoutInterp = NULL;
static int GUSISpinFunction _ANSI_ARGS_((spin_msg msg, long param));
int 	TclMacConvertEvent _ANSI_ARGS_((EventRecord *eventPtr));
int 	TkMacConvertEvent _ANSI_ARGS_((EventRecord *eventPtr));

/*
 *	The following four functions provide the UI for the console package.
 *	Users wishing to replace SIOUX with their own console package need
 *	only provide the four functions below in a library.
 */

/*
 *	extern short InstallConsole(short fd);
 *
 *	Installs the Console package, this function will be called right
 *	before any read or write to one of the standard streams.
 *
 *	short fd:		The stream which we are reading/writing to/from.
 *	returns short:	0 no error occurred, anything else error.
 */

short InstallConsole(short fd)
{
#pragma unused (fd)

	return 0;
}

/*
 *	extern void RemoveConsole(void);
 *
 *	Removes the console package.  It is called after all other streams
 *	are closed and exit functions (installed by either atexit or _atexit)
 *	have been called.  Since there is no way to recover from an error,
 *	this function doesn't need to return any.
 */

void RemoveConsole(void)
{
}

/*
 *	extern long WriteCharsToConsole(char *buffer, long n);
 *
 *	Writes a stream of output to the Console window.  This function is
 *	called by write.
 *
 *	char *buffer:	Pointer to the buffer to be written.
 *	long n:			The length of the buffer to be written.
 *	returns short:	Actual number of characters written to the stream,
 *					-1 if an error occurred.
 */

long WriteCharsToConsole(char *buffer, long n)
{
    TkConsolePrint(gStdoutInterp, buffer, n);
	return n;
}

/*
 *	extern long ReadCharsFromConsole(char *buffer, long n);
 *
 *	Reads from the Console into a buffer.  This function is called by
 *	read.
 *
 *	char *buffer:	Pointer to the buffer which will recieve the input.
 *	long n:			The maximum amount of characters to be read (size of
 *					buffer).
 *	returns short:	Actual number of characters read from the stream,
 *					-1 if an error occurred.
 */

long ReadCharsFromConsole(char *buffer, long n)
{
#pragma unused (buffer, n)

	return 0;
}

/*
 *	extern char *__ttyname(long fildes);
 *
 *	Return the name of the current terminal (only valid terminals are
 *	the standard stream (ie stdin, stdout, stderr).
 *
 *	long fildes:	The stream to query.
 *
 *	returns char*:	A pointer to static global data which contains a C
 *                      string or NULL if the stream is not valid.
 */

extern char *__ttyname(long fildes)
{
#pragma unused (fildes)
	/* all streams have the same name */
	static char *__devicename = "null device";

	if (fildes >= 0 && fildes <= 2)
		return (__devicename);

	return (0L);
}

short
SIOUXHandleOneEvent(EventRecord *event)
{
    return 0;
}

void
tkMacInstallMWConsole(Tcl_Interp *interp)
{
    gStdoutInterp = interp;
}

/*
 *----------------------------------------------------------------------
 *
 * TclMacConsoleInit --
 *
 *	This procedure calls initalization routines to set up a simple
 *	console on a Macintosh.  This is necessary as the Mac doesn't
 *	have a stdout & stderr by default.
 *
 * Results:
 *	Returns TCL_OK if everything went fine.  If it didn't the 
 *	application should probably fail.
 *
 * Side effects:
 *	Inits the appropiate console package.
 *
 *----------------------------------------------------------------------
 */

int
TclMacConsoleInit()
{
    GUSIDefaultSetup();
    GUSISetHook(GUSI_SpinHook, (GUSIHook) GUSISpinFunction);

    return TCL_OK;
}

/*
 *----------------------------------------------------------------------
 *
 * GUSISpinFunction --
 *
 *	Spins.
 *
 * Results:
 *	Returns .
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

static int
GUSISpinFunction(msg, param)
    spin_msg msg;
    long param;
{
    EventRecord ev;
    
    WaitNextEvent(0, &ev, 0, NULL);
    return 0;
}

/*
 *----------------------------------------------------------------------
 *
 * TclMacConvertEvent --
 *
 *	This function handles a macintosh event.  In a Tcl interperter
 *	all we need to do is pass the event to SIOUX and we are done.
 *
 * Results:
 *	True if event was handled, false otherwise.
 *
 * Side effects:
 *	Passes the event to SIOUX.
 *
 *----------------------------------------------------------------------
 */

int
TclMacConvertEvent(eventPtr)
    EventRecord *eventPtr;
{
    return TkMacConvertEvent(eventPtr);
}

/*
 *----------------------------------------------------------------------
 *
 * TclGeneratePollingEvents --
 *
 *	Here we just call through to the Tk procedure for checking
 *	for motion and enter & leave events.
 *
 * Results:
 *	The result of TkGeneratePollingEvents.
 *
 * Side effects:
 *	None.
 *
 *----------------------------------------------------------------------
 */

int
TclGeneratePollingEvents()
{
    return TkGeneratePollingEvents();
}

